#!/usr/bin/env python

from agate.aggregations.has_nulls import HasNulls
from agate.computations.base import Computation
from agate.data_types import Text
from agate.exceptions import DataTypeError
from agate.utils import slugify, issequence


class Slug(Computation):
    """
    Convert text values from one or more columns into slugs. If multiple column
    names are given, values from those columns will be appended in the given
    order before standardizing.

    :param column_name:
        The name of a column or a sequence of column names containing
        :class:`.Text` values.
    :param ensure_unique:
        If True, any duplicate values will be appended with unique identifers.
        Defaults to False.
    """
    def __init__(self, column_name, ensure_unique=False, **kwargs):
        self._column_name = column_name
        self._ensure_unique = ensure_unique
        self._slug_args = kwargs

    def get_computed_data_type(self, table):
        return Text()

    def validate(self, table):
        if issequence(self._column_name):
            column_names = self._column_name
        else:
            column_names = [self._column_name]

        for column_name in column_names:
            column = table.columns[column_name]

            if not isinstance(column.data_type, Text):
                raise DataTypeError('Slug column must contain Text data.')

            if HasNulls(column_name).run(table):
                raise ValueError('Slug column cannot contain `None`.')

    def run(self, table):
        """
        :returns:
            :class:`string`
        """
        new_column = []

        for row in table.rows:
            if issequence(self._column_name):
                column_value = ''
                for column_name in self._column_name:
                    column_value = column_value + ' ' + row[column_name]

                new_column.append(column_value)
            else:
                new_column.append(row[self._column_name])

        return slugify(new_column, ensure_unique=self._ensure_unique, **self._slug_args)



#!/usr/bin/env python

from datetime import date, datetime, time

import isodate
import parsedatetime
import six

from agate.data_types.base import DataType
from agate.exceptions import CastError


ZERO_DT = datetime.combine(date.min, time.min)


class Date(DataType):
    """
    Data representing dates alone.

    :param date_format:
        A formatting string for :meth:`datetime.datetime.strptime` to use
        instead of using regex-based parsing.
    """
    def __init__(self, date_format=None, **kwargs):
        super(Date, self).__init__(**kwargs)

        self.date_format = date_format
        self.parser = parsedatetime.Calendar(version=parsedatetime.VERSION_CONTEXT_STYLE)

    def __getstate__(self):
        """
        Return state values to be pickled. Exclude _parser because parsedatetime
        cannot be pickled.
        """
        odict = self.__dict__.copy()
        del odict['parser']
        return odict

    def __setstate__(self, data):
        """
        Restore state from the unpickled state values. Set _parser to an instance
        of the parsedatetime Calendar class.
        """
        self.__dict__.update(data)
        self.parser = parsedatetime.Calendar(version=parsedatetime.VERSION_CONTEXT_STYLE)

    def cast(self, d):
        """
        Cast a single value to a :class:`datetime.date`.

        :param date_format:
            An optional :func:`datetime.strptime` format string for parsing
            datetimes in this column.
        :returns:
            :class:`datetime.date` or :code:`None`.
        """
        if type(d) is date or d is None:
            return d
        elif isinstance(d, six.string_types):
            d = d.strip()

            if d.lower() in self.null_values:
                return None
        else:
            raise CastError('Can not parse value "%s" as date.' % d)

        if self.date_format:
            try:
                dt = datetime.strptime(d, self.date_format)
            except:
                raise CastError('Value "%s" does not match date format.' % d)

            return dt.date()

        try:
            (value, ctx, _, _, matched_text), = self.parser.nlp(d, sourceTime=ZERO_DT)
        except (TypeError, ValueError):
            raise CastError('Value "%s" does not match date format.' % d)
        else:
            if matched_text == d and ctx.hasDate and not ctx.hasTime:
                return value.date()

        raise CastError('Can not parse value "%s" as date.' % d)

    def csvify(self, d):
        if d is None:
            return None

        return d.isoformat()

    def jsonify(self, d):
        return self.csvify(d)



#!/usr/bin/env python

from agate.aggregations.base import Aggregation
from agate.data_types import Number
from agate.exceptions import DataTypeError


class Sum(Aggregation):
    """
    Calculate the sum of a column.

    :param column_name:
        The name of a column containing :class:`.Number` data.
    """
    def __init__(self, column_name):
        self._column_name = column_name

    def get_aggregate_data_type(self, table):
        return Number()

    def validate(self, table):
        column = table.columns[self._column_name]

        if not isinstance(column.data_type, Number):
            raise DataTypeError('Sum can only be applied to columns containing Number data.')

    def run(self, table):
        column = table.columns[self._column_name]

        return sum(column.values_without_nulls())



#!/usr/bin/env python

try:
    import unittest2 as unittest
except ImportError:
    import unittest

import agate


class AgateTestCase(unittest.TestCase):
    """
    Unittest case for quickly asserting logic about tables.
    """
    def assertColumnNames(self, table, names):
        """
        Verify the column names in the given table match what is expected.
        """
        self.assertIsInstance(table, agate.Table)

        self.assertSequenceEqual(table.column_names, names)
        self.assertSequenceEqual(
            [c.name for c in table.columns],
            names
        )

        for row in table.rows:
            self.assertSequenceEqual(
                row.keys(),
                names
            )

    def assertColumnTypes(self, table, types):
        """
        Verify the column types in the given table are of the expected types.
        """
        self.assertIsInstance(table, agate.Table)

        table_types = table.column_types
        column_types = [c.data_type for c in table.columns]

        for i, test_type in enumerate(types):
            self.assertIsInstance(table_types[i], test_type)
            self.assertIsInstance(column_types[i], test_type)

    def assertRows(self, table, rows):
        """
        Verify the row data in the given table match what is expected.
        """
        self.assertIsInstance(table, agate.Table)

        for i, row in enumerate(rows):
            self.assertSequenceEqual(table.rows[i], row)

    def assertRowNames(self, table, names):
        """
        Verify the row names in the given table match what is expected.
        """
        self.assertIsInstance(table, agate.Table)

        self.assertSequenceEqual(table.row_names, names)
        self.assertSequenceEqual(
            table.rows.keys(),
            names
        )

        for column in table.columns:
            self.assertSequenceEqual(
                column.keys(),
                names
            )



#!/usr/bin/env python
# pylint: disable=W0212


def find(self, test):
    """
    Find the first row that passes a test.

    :param test:
        A function that takes a :class:`.Row` and returns :code:`True` if
        it matches.
    :type test:
        :class:`function`
    :returns:
        A single :class:`.Row` if found, or `None`.
    """
    for row in self._rows:
        if test(row):
            return row

    return None



#!/usr/bin/env python

"""
Computations create a new value for each :class:`.Row` in a :class:`.Table`.
When used with :meth:`.Table.compute` these new values become a new column.
For instance, the :class:`.PercentChange` computation takes two column names as
arguments and computes the percentage change between them for each row.

There are a variety of basic computations, such as :class:`.Change` and
:class:`.Percent`. If none of these meet your needs you can use the
:class:`Formula` computation to apply an arbitrary function to the row.
If this still isn't flexible enough, it's simple to create a custom computation
class by inheriting from :class:`Computation`.
"""

from agate.computations.base import Computation  # noqa

from agate.computations.formula import Formula  # noqa
from agate.computations.change import Change  # noqa
from agate.computations.percent import Percent  # noqa
from agate.computations.percent_change import PercentChange  # noqa
from agate.computations.rank import Rank  # noqa
from agate.computations.percentile_rank import PercentileRank  # noqa
from agate.computations.slug import Slug  # noqa



#!/usr/bin/env python

import six

from agate.aggregations import *
from agate.data_types import *
from agate.columns import Column  # noqa
from agate.computations import *
from agate.config import get_option, set_option, set_options  # noqa
from agate.exceptions import *
# import agate.fixed as fixed  # noqa
from agate.mapped_sequence import MappedSequence  # noqa
from agate.rows import Row  # noqa
from agate.table import Table  # noqa
from agate.tableset import TableSet  # noqa
from agate.testcase import AgateTestCase  # noqa
from agate.type_tester import TypeTester  # noqa
from agate.utils import *
from agate.warns import NullCalculationWarning, DuplicateColumnWarning, warn_null_calculation, warn_duplicate_column  # noqa

if six.PY2:  # pragma: no cover
    import agate.csv_py2 as csv  # noqa
else:
    import agate.csv_py3 as csv  # noqa
